import warnings

__version__ = "0.2.3"

warnings.warn(
    "iterable_etl is deprecated and will no longer be maintained. Consider using an appropriate ETL tool for this functionality.",
    FutureWarning,
)



"""dbg"""

import os
from typing import Any, Callable, TypeVar

import pandas as pd
from loguru import logger

T = TypeVar("T")


def dbg(__message: str, *args: Any, **kwargs: Any) -> None:
    """dbg"""
    if os.environ.get("APP_ENV") == "development":
        logger.debug(__message, *args, **kwargs)


def error_handler_decorator(func: Callable[..., T]) -> Callable[..., T]:
    def wrapper(*args: Any, **kwargs: Any):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            dbg(f"An exception occurred: {e}")

    return wrapper


def print_dataframe_head(func: Callable[..., T]) -> Callable[..., T]:
    def wrapper(*args: Any, **kwargs: Any) -> T:
        result = func(*args, **kwargs)
        if os.environ.get("APP_ENV") == "development":
            if isinstance(result, pd.DataFrame):
                dbg(func.__name__)
                dbg(result.head(10))
                dbg("")
            else:
                dbg("Returned object is not a DataFrame.")
        return result

    return wrapper


def write_dataframe_to_csv(file_name: str):
    def _write_dataframe_to_csv(func: Callable[..., T]) -> Callable[..., T]:
        def wrapper(*args: Any, **kwargs: Any) -> T:
            result = func(*args, **kwargs)
            if os.environ.get("SAMPLE_OUTPUT") == "True":
                if isinstance(result, pd.DataFrame):
                    directory = "sample_output"
                    os.makedirs(directory, exist_ok=True)
                    csv_filename = f"{file_name}.csv"
                    csv_path = os.path.join(directory, csv_filename)
                    result.to_csv(csv_path, index=False)
                    dbg(f"DataFrame saved to: {csv_path}")
                else:
                    dbg("Returned object is not a DataFrame.")
            return result

        return wrapper

    return _write_dataframe_to_csv



"""user_history"""

from datetime import datetime
from typing import Dict

from pandas import DataFrame as PandasDF

from iterable_etl.libs.cnst import get_headers, urls
from iterable_etl.libs.dbg import print_dataframe_head, write_dataframe_to_csv
from iterable_etl.libs.network import get_data_csv
from iterable_etl.libs.transform import csv_to_dataframe


def get_export_data(api_url: str, headers: Dict[str, str]) -> str:
    """Make a GET request to the Iterable API and return a csv string of export data."""
    data = get_data_csv(api_url, headers)
    return data


def create_only_fields(onlyFields: str) -> str:
    string = "&".join(f"onlyFields={field}" for field in onlyFields)
    return string


@write_dataframe_to_csv("user_history")
@print_dataframe_head
def user_history_df(
    start_date: str,
    end_date: str,
    fields_list=[
        "email",
        "emailListIds",
        "firstName",
        "lastName",
        "phoneNumber",
        "phoneNumberDetails.carrier",
        "phoneNumberDetails.countryCodeISO",
        "phoneNumberDetails.lineType",
        "phoneNumberDetails.updatedAt",
        "signupDate",
        "signupSource",
        "userId",
        "profileUpdatedAt",
    ],
) -> PandasDF:
    """user_history dataframe"""

    base_url = urls["export"]

    fields_to_query = create_only_fields(fields_list)

    url = f"{base_url}?dataTypeName=user&range=All&startDateTime={start_date}&endDateTime={end_date}&{fields_to_query}"

    data = get_export_data(url, get_headers())

    df = csv_to_dataframe(data)

    now = datetime.now()
    df["meta"] = str(
        {
            "start_date": start_date,
            "end_date": end_date,
            "url": url,
            "requested_on": now,
        }
    )
    return df



"""spark"""

from pandas import DataFrame as PandasDF
from pyspark.sql import DataFrame as SparkDF
from pyspark.sql import SparkSession

from iterable_etl.libs.dbg import dbg


def dataframe_to_spark(df: PandasDF, spark: SparkSession) -> SparkDF:
    """
    Convert the Pandas DataFrame to a Spark DataFrame.

    from pyspark.sql import SparkSession
    spark = SparkSession.builder.getOrCreate()
    """
    spark_df = spark.createDataFrame(df)
    return spark_df


def write_to_databricks_table(spark_df: SparkDF, table_name: str) -> None:
    """
    Write the Spark DataFrame to a Databricks table.
    """
    spark_df.write.mode("overwrite").saveAsTable(table_name)


def write_diff_for_list_user_history(
    spark: SparkSession, pandas_df: PandasDF, target_schema_table_name: str
):
    """
    Write entries to `list_user_history` table WO duplication
    """

    spark_df = spark.createDataFrame(pandas_df)

    spark_df.createOrReplaceTempView("temp_table")

    existing_entries = spark.sql(
        """
        SELECT t1.*
        FROM temp_table t1
        JOIN {} t2 ON t1.list_id = t2.list_id
    """.format(
            target_schema_table_name
        )
    )

    if existing_entries.count() == 0:
        spark_df.write.format("delta").mode("append").saveAsTable(
            target_schema_table_name
        )
    else:
        dbg("Skipping write due to overlapping entries.")



"""cli"""
import click

from iterable_etl.tables.campaign_history import campaign_history_df
from iterable_etl.tables.campaign_list_history import campaign_list_history_df
from iterable_etl.tables.campaign_metrics import campaign_metrics_df
from iterable_etl.tables.list import list_df
from iterable_etl.tables.list_user_history import list_user_history_df
from iterable_etl.tables.user_history import user_history_df


@click.command()
@click.option("--table")
def main(table: str) -> None:
    """program"""
    if table == "campaign_history":
        campaign_history_df()
    elif table == "campaign_metrics":
        campaign_metrics_df()
    elif table == "list":
        list_df()
    elif table == "campaign_list_history":
        campaign_list_history_df()
    elif table == "list_user_history":
        list_user_history_df()
    elif table == "user_history":
        start_date = "2023-06-25"
        end_date = "2023-06-27"
        user_history_df(start_date=start_date, end_date=end_date)
    elif table == "ALL":
        campaign_history_df()
        campaign_metrics_df()
        list_df()
        campaign_list_history_df()
        list_user_history_df()
    else:
        raise ValueError("Invalid table selection")



"""list"""

from typing import Any, Dict

from pandas import DataFrame as PandasDF

from iterable_etl.libs.cnst import get_headers, urls
from iterable_etl.libs.dbg import print_dataframe_head, write_dataframe_to_csv
from iterable_etl.libs.network import get_data_json
from iterable_etl.libs.transform import json_to_dataframe


def get_list_data(api_url: str, headers: Dict[str, str]) -> Dict[str, Any]:
    """Make a GET request to the Iterable API and return a dictionary of list data."""
    data = get_data_json(api_url, headers)
    return data["lists"]


@write_dataframe_to_csv("list")
@print_dataframe_head
def list_df() -> PandasDF:
    """list dataframe"""
    data = get_list_data(urls["lists"], get_headers())
    df = json_to_dataframe(data)
    return df



"""constants"""

import os
from datetime import date
from typing import Any, Dict

urls = {
    "lists": "https://api.iterable.com/api/lists",
    "campaigns": "https://api.iterable.com/api/campaigns",
    "metrics": "https://api.iterable.com/api/campaigns/metrics",
    "lists_get_users": "https://api.iterable.com/api/lists/getUsers?listId=",
    "export": "https://api.iterable.com/api/export/data.csv",
}


def get_headers() -> Dict[str, Any]:
    """header config for Iterable API"""
    headers = {
        "Api-Key": os.environ.get("ITERABLE_KEY"),
        "Content-Type": "application/json",
    }
    return headers


def get_todays_date():
    """today"""
    today = date.today()
    return today.strftime("%Y-%m-%d")



"""transform"""

import io
import re
from typing import Any, Dict

import pandas as pd
from pandas import DataFrame as PandasDF


def json_to_dataframe(data: Dict[str, Any]) -> PandasDF:
    """Convert the JSON data to a Pandas DataFrame."""
    df = pd.json_normalize(data)
    return df


def csv_to_dataframe(data: str) -> PandasDF:
    """Convert the CSV data to a Pandas DataFrame."""
    df = pd.read_csv(io.StringIO(data))
    return df


def clean_column_headers(df: PandasDF) -> PandasDF:
    """Remove special characters and spaces, columns as camelCase."""
    clean_headers = []
    for column in df.columns:
        clean_header = re.sub(r"[^a-zA-Z0-9_ ]", "", column.lower())
        words = clean_header.split()
        clean_header = "".join(word.title() for word in words)
        clean_headers.append(clean_header)

    # Assign the clean headers to the DataFrame
    df.columns = clean_headers

    return df



"""list_user_history"""
import time
from typing import Any, Dict, List

import pandas as pd
from pandas import DataFrame as PandasDF

from iterable_etl.libs.cnst import get_headers, urls
from iterable_etl.libs.dbg import (
    dbg,
    error_handler_decorator,
    print_dataframe_head,
    write_dataframe_to_csv,
)
from iterable_etl.libs.network import get_data_json, get_data_text
from iterable_etl.libs.transform import csv_to_dataframe


def get_list_ids(api_url: str, headers: Dict[str, str]) -> List[Any]:
    """Make a GET request to the Iterable API and return a dictionary of list data."""
    data = get_data_json(api_url, headers)
    list_ids = [_list["id"] for _list in data["lists"]]
    list_ids.sort()
    return list_ids


def default_diff_writer():
    dbg("no diff writer")


# spark: SparkSession, pandas_df: PandasDF, target_schema_table_name: str


@write_dataframe_to_csv("list_user_history")
@print_dataframe_head
@error_handler_decorator
def list_user_history_df(
    spark=None, target_schema_table_name=None, diff_writer=None
) -> PandasDF:
    """
    list_user_history dataframe

    This will take ~12 hours to run. It should be setup as an individual job.

    TODO: Further diffing work to make the run less jank.

    """

    list_user_history_url = urls["lists_get_users"]

    df = pd.DataFrame()

    list_ids = get_list_ids(urls["lists"], get_headers())

    for i, list_id in enumerate(list_ids):
        dbg(
            "user list for list id {} of {}, list_id= {}".format(
                i, len(list_ids), list_id
            )
        )
        url = f"{list_user_history_url}{list_id}"

        response_text = get_data_text(url, get_headers())

        time.sleep(13)

        if response_text:
            dbg("list_id={} has content".format(list_id))
            _df = csv_to_dataframe(response_text)
            _df = _df.rename(columns={_df.columns[0]: "user_email"})
            _df["list_id"] = list_id
            if (
                spark is not None
                and target_schema_table_name is not None
                and diff_writer is not None
                and _df.shape[0] > 0
            ):
                diff_writer(spark, _df, target_schema_table_name)
            df = pd.concat([df, _df])

    return df



"""campaign_list_history"""

from typing import Any, Dict, List

import pandas as pd
from pandas import DataFrame as PandasDF

from iterable_etl.libs.cnst import get_headers, urls
from iterable_etl.libs.dbg import print_dataframe_head, write_dataframe_to_csv
from iterable_etl.libs.network import get_data_json


def get_campaign_data(api_url: str, headers: Dict[str, str]) -> List[Dict[str, Any]]:
    """Make a GET request to the Iterable API and return a dictionary of campaigns data."""
    data = get_data_json(api_url, headers)
    return data["campaigns"]


def explode_list_ids(data: List[Dict[str, Any]]) -> PandasDF:
    """flatten listIds and associate with campaignId"""
    df_data = []
    for campaign in data:
        if "listIds" in campaign:
            for list_id in campaign["listIds"]:
                df_data.append(
                    {
                        "campaignId": campaign["id"],
                        "listId": list_id,
                        "updatedAt": campaign["updatedAt"],
                    }
                )

    df = pd.DataFrame(df_data)

    return df


@write_dataframe_to_csv("campaign_list_history")
@print_dataframe_head
def campaign_list_history_df() -> PandasDF:
    """campaign_history_dataframe"""
    data = get_campaign_data(urls["campaigns"], get_headers())
    df = explode_list_ids(data)

    return df



"""network"""

from typing import Any, Dict
from urllib.parse import urlparse, urlunparse

import requests
from requests import Response

from iterable_etl.libs.dbg import dbg


def trim_string(string: str):
    if len(string) > 150:
        trimmed_string = string[:150].strip() + "..."
        return trimmed_string
    else:
        return string


def get_data(api_url: str, headers: Dict[str, str]) -> Response:
    """Make a GET request to the Iterable API and return the data."""
    dbg_url = trim_string(api_url)
    dbg("Making request to {api_url}", api_url=dbg_url)
    response = requests.get(api_url, headers=headers, timeout=120)
    dbg("{api_url} response code {code}", api_url=dbg_url, code=response.status_code)
    response.raise_for_status()
    return response


def get_data_json(api_url: str, headers: Dict[str, str]) -> Dict[str, Any]:
    data = get_data(api_url, headers)
    return data.json()


def get_data_text(api_url: str, headers: Dict[str, str]) -> str:
    data = get_data(api_url, headers)
    return data.text


def get_data_csv(api_url: str, headers: Dict[str, str]) -> str:
    data = get_data(api_url, headers)
    content = data.content.decode("utf-8")
    return content



#!/usr/bin/env python

from setuptools import setup, find_packages

with open("README_PUBLIC.md", "r") as fh:
    long_description = fh.read()

setup(
    author="Neo Financial",
    author_email="engineering@neofinancial.com",
    python_requires=">=3.6",
    name="iterable_etl",
    version="0.2.3",
    description="Replicate iterable data in databricks",
    classifiers=[
        "Development Status :: 7 - Inactive",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
    ],
    license="UNLICENSED",
    packages=find_packages(include=["iterable_etl", "iterable_etl.*"]),
    url="https://github.com/neofinancial/iterable_etl",
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
        "click",
        "pandas>=1,<2",
        "requests",
        "pyspark",
        "loguru",
        "python-dotenv",
        "typing_extensions",
    ],
    tests_require=["pytest"],
)



"""campaign_metrics"""
import time
from typing import Dict

import pandas as pd
from pandas import DataFrame as PandasDF

from iterable_etl.libs.cnst import get_headers, get_todays_date, urls
from iterable_etl.libs.dbg import print_dataframe_head, write_dataframe_to_csv
from iterable_etl.libs.network import get_data_json, get_data_text
from iterable_etl.libs.transform import clean_column_headers, csv_to_dataframe


def get_campaign_ids(api_url: str, headers: Dict[str, str]) -> list[int]:
    """Make a GET request to the Iterable API and return a list of campaign ids."""
    data = get_data_json(api_url, headers)
    campaign_ids = [campaign["id"] for campaign in data["campaigns"]]
    return campaign_ids


@write_dataframe_to_csv("campaign_metrics")
@print_dataframe_head
def campaign_metrics_df() -> PandasDF:
    """campaign_metrics dataframe"""

    campaign_metrics_url = urls["metrics"]

    start_date = "2021-06-01"
    end_date = get_todays_date()

    batch_size = 300

    df = pd.DataFrame()

    campaign_ids = get_campaign_ids(urls["campaigns"], get_headers())

    campaign_id_batches = [
        campaign_ids[i : i + batch_size]
        for i in range(0, len(campaign_ids), batch_size)
    ]

    for campaign_id_batch in campaign_id_batches:
        campaign_id_params = "&".join(
            [f"campaignId={campaign_id}" for campaign_id in campaign_id_batch]
        )

        url = f"{campaign_metrics_url}?{campaign_id_params}&startDateTime={start_date}&endDateTime={end_date}"

        response_text = get_data_text(url, get_headers())

        time.sleep(11)

        data = csv_to_dataframe(response_text)
        df = pd.concat([df, data])

    df = clean_column_headers(df)
    return df
